#include "printf.h"
#include "as31glue.h"
#include "parse.h"	// for event codes
#include "string.h"
#include "stricmp.h"

#include "display.h"

#define SCROLL_SPEED 5
#define VOL_TIMEOUT 50

xdata int xwrap=0;	// Position inside filebuf
xdata unsigned char xscroll=0;	// Scroll ?

xdata unsigned char filebuf[128];
xdata unsigned char displaybuf[25]="                        ";

extern unsigned char current_button;

//
// Timer usage:
//
// timer 0 - scrolling song name
// timer 1 - Bottom line display timeout
// timer 2
// timer 3
//

/* handle user interface events that related to navigation of */
/* the on-screen information */

void lcd_user_action(unsigned char event)
{
	switch(event) {

	  case E_INC:
		add_new_event(E_VOL_UP);
		break;

	  case E_DEC:
		add_new_event(E_VOL_DOWN);
		break;

	  default:
		printf("Unsupported LCD Navigation, #%d\r\n", event);
	}
}





void lcd_display_filename(char *filename)
//
//	lcd_display_filename
//
//	Takes long file name.  Strips last 4 characters if
//	they are ".mp3".  Pads with a few spaces and
//	fills to 24 characters if shorter than that.
//
{
	unsigned char z;

	// Kill any pending timer0 interrupts
	clear_timer(0);
	xscroll=0;

	strcpy(filebuf,filename);
	if(stricmp(&filebuf[strlen(filebuf)-4],".MP3")==0)
	{
		filebuf[strlen(filebuf)-4]='\0';
	}

	for(z=0;z<strlen(filebuf);z++)
		if(filebuf[z]=='_')
			filebuf[z]=' ';

	if(strlen(filebuf)<24)
	{
		while(strlen(filebuf)<24) strcat(filebuf," ");
		xscroll=0;
	}
	else
	{
		strcat(filebuf,"   ");
		xscroll=1;
	}

	for(z=0;z<24;z++) displaybuf[z]=filebuf[z];
	displaybuf[24]='\0';
	xwrap=23;

	printf("\\[\\A0\\B @%s\\]",displaybuf);

	// Set timer to scroll display
	if(xscroll==1)
		set_timer(0,SCROLL_SPEED);
}





void lcd_display_update(void)
//
//	lcd_display_update
//
//	Called when timer 0 expires.
//	Displays displaybuf on the top line of the display, then
//	scrolls displaybuf to the left, and grabs the next
//	character from filebuf.
//
{
	unsigned char z;

	if(xscroll==0)
		printf("Got here in error!!!\n");

	printf("\\[\\A0\\B @%s\\]\r\n",displaybuf);

	xwrap++;
	if(xwrap>=strlen(filebuf)) xwrap=0;

	for(z=0;z<23;z++) displaybuf[z]=displaybuf[z+1];

	displaybuf[23]=filebuf[xwrap];

	if(xscroll==1)
		set_timer(0,SCROLL_SPEED);
}





void lcd_volume_set(unsigned char attn)
//
//	lcd_volume_set
//
//	Displays a volume bargraph on the bottom line of
//	the display.
//
{
	unsigned char z,x;

	// Make the bargraph fit in 17 characters
	x=(0x40-attn)/4;

	printf("\\[\\A1\\B 'Vol: [");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			printf("\\C!\\w\\C ");
		}
		else
		{
			printf("-");
		}
	}
	printf("]\\]");
	set_timer(1,50);
}


void lcd_bass_set(char bass)
//
//	lcd_bass_set
//
//	Displays a bass bargraph on the bottom line of
//	the display.
//
//	The sta013 expects a 2s compliment number from
//	-12 to 12 corresponding to a bass boost of
//	-18dB to 18dB.
//
{
	unsigned char z,x;

	// Make the bargraph fit in 17 characters
	x=(bass+12)*17/25;

	printf("\\[\\A1\\B 'Bas: [");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			printf("\\C!\\w\\C ");
		}
		else
		{
			printf("-");
		}
	}
	printf("]\\]");
	set_timer(1,50);
}


void lcd_treble_set(char treble)
//
//	lcd_treble_set
//
//	Displays a treble bargraph on the bottom line of
//	the display.
//
//	The sta013 expects a 2s compliment number from
//	-12 to 12 corresponding to a treble boost of
//	-18dB to 18dB.
//
{
	unsigned char z,x;

	// Make the bargraph fit in 17 characters
	x=(treble+12)*17/25;

	printf("\\[\\A1\\B 'Tre: [");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			printf("\\C!\\w\\C ");
		}
		else
		{
			printf("-");
		}
	}
	printf("]\\]");
	set_timer(1,50);
}




void lcd_clear_bottom_row(void)
//
//	lcd_clear_bottom_row
//
//	Erases all text on the bottom row of the
//	display.  Set current button to nothing.
//
{
	printf("\\[\\A1\\B '                        \\]");
	current_button=0;
}


void lcd_rand_set(unsigned char is_rand_mode)
{
	if(is_rand_mode)
	{
		printf("\\[\\A1\\B 'Random Play On          \\]");
	}
	else
	{
		printf("\\[\\A1\\B 'Random Play Off         \\]");
	}
	set_timer(1,50);
}




void lcd_setup(void)
{
	printf("lcd:");
	printf("\\H0INC         ");	// button 0 down = INC
	printf("\\H@            ");	// button 0 up = <nothing>
	printf("\\HPINC         ");	// button 0 repeat = INC

	printf("\\H1DEC         ");	// button 1 down = INC
	printf("\\HA            ");	// button 1 up = <nothing>
	printf("\\HQDEC         ");	// button 1 repeat = INC

	printf("\\H2PRV_LIST    ");	// button 2 down = PRV_PLAYLIST
	printf("\\HB            ");	// button 2 up = <nothing>
	printf("\\HRPRV_LIST    ");	// button 2 repeat = PRV_PLAYLIST

	printf("\\H3NXT_LIST    ");	// button 3 down = NXT_PLAYLIST
	printf("\\HC            ");	// button 3 up = <nothing>
	printf("\\HSNXT_LIST    ");	// button 3 repeat = NXT_PLAYLIST

	printf("\\H4            ");	// button 4 down = <nothing>
	printf("\\HDPREVIOUS    ");	// button 4 up = PREVIOUS
	printf("\\HTREVERSE     ");	// button 4 repeat = REVERSE

	printf("\\H5PLAY        ");	// button 5 down = PLAY
	printf("\\HE            ");	// button 5 up = <nothing>
	printf("\\HU            ");	// button 5 repeat = <nothing>

	printf("\\H6            ");	// button 6 down = <nothing>
	printf("\\HFNEXT        ");	// button 6 up = NEXT
	printf("\\HVFORWARD     ");	// button 6 repeat = FORWARD

	printf("\\H7UP          ");	// button 7 down = UP
	printf("\\HG            ");	// button 7 up = <nothing>
	printf("\\HWUP          ");	// button 7 repeat = UP

	printf("\\H8LEFT        ");	// button 8 down = LEFT
	printf("\\HH            ");	// button 8 up = <nothing>
	printf("\\HXLEFT        ");	// button 8 repeat = LEFT

	printf("\\H9RIGHT       ");	// button 9 down = RIGHT
	printf("\\HI            ");	// button 9 up = <nothing>
	printf("\\HYRIGHT       ");	// button 9 repeat = RIGHT

	printf("\\H:DOWN        ");	// button 10 down = DOWN
	printf("\\HJ            ");	// button 10 up = <nothing>
	printf("\\HZDOWN        ");	// button 10 repeat = DOWN

	printf("\\H;ENTER       ");	// button 11 down = ENTER
	printf("\\HK            ");	// button 11 up = <nothing>
	printf("\\H[ENTER       ");	// button 11 repeat = ENTER

	printf("\r\n");


}


